/*
*   Copyright 2011 by Eugeni Kozhanov <eugeni.kozhanov@rosalab.ru>
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU Library General Public License version 2,
*   or (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details
*
*   You should have received a copy of the GNU Library General Public
*   License along with this program; if not, write to the
*   Free Software Foundation, Inc.,
*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "configpanel.h"

#include <limits>

#include <QDesktopWidget>
#include <QAction>
#include <QApplication>
#include <QGraphicsLinearLayout>
#include <QGraphicsLayout>
#include <QGraphicsSceneDragDropEvent>
#include <QVBoxLayout>
#include <QTimer>
#include <QPainter>
#include <QLabel>

#include <KDebug>
#include <KIcon>
#include <KIconLoader>
#include <kwindowsystem.h>                                                                                                                                                     
#include <netwm.h>  

#include <Plasma/Corona>
#include <Plasma/FrameSvg>
#include <Plasma/Theme>
#include <Plasma/AbstractToolBox>
#include <Plasma/View>
#include <Plasma/PaintUtils>
#include <Plasma/WindowEffects>

//#include <kephal/screens.h>
#include <Plasma/Separator>


using namespace Plasma;


ConfigPanel::ConfigPanel(QWidget *p)
    //: QWidget(parent)
{
	QRect screenRect = QApplication::desktop()->screenGeometry();

	i_width = screenRect.width();
	i_height = screenRect.height();
	
	setFixedSize(screenRect.width(),50);
	setAttribute(Qt::WA_DeleteOnClose);                                                                                                                                        
	setFocus(Qt::ActiveWindowFocusReason);
	//setAttribute(Qt::WA_TranslucentBackground);  
	
	 
        move(0,screenRect.height() - 50);
	setWindowFlags(Qt::FramelessWindowHint | Qt::WindowStaysOnTopHint);
        //m_mainConfiguration->setAcceptDrops(true);
	KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager | NET::Sticky | NET::KeepAbove);
	
	QHBoxLayout *m_clayout = new QHBoxLayout();
	
	//m_clayout->setSize(size());
	m_clayout->setAlignment(Qt::AlignHCenter);
	m_clayout->setContentsMargins(0, 0, 0, 0); 
    
    
	m_sizeTool = new QToolButton(this);
	m_sizeTool->setText(i18n("Height"));
	m_sizeTool->resize(size());
	//m_sizeTool->setCheckable(false);
	//m_sizeTool->setAutoExclusive(false);
	//m_sizeTool->installEventFilter(this);
	setCursor(Qt::SizeVerCursor);
	
	//QLabel label(i18n("Height"));
	
	m_clayout->addWidget(m_sizeTool,Qt::AlignHCenter);
	//m_clayout->addStretch();
    
	setLayout(m_clayout);
	
	connect(KWindowSystem::self(), SIGNAL(activeWindowChanged(WId)), this, SLOT(onActiveWindowChanged(WId)));
	
	b_drug = false;
	i_startDrug = screenRect.height() - 50;
}

/*
ConfigPanel::~ConfigPanel()
{
	
}
*/

void ConfigPanel::mousePressEvent(QMouseEvent *event)
{
	//if(event->button() == Qt::LeftButton){
	    i_startDrug = event->globalY();
	    b_drug = true;
	//}
}

void ConfigPanel::mouseReleaseEvent(QMouseEvent *event)
{
	
	//if((event->buttons() == Qt::LeftButton) /*&& b_drug*/){
		b_drug = false;
	//}
}

void ConfigPanel::mouseMoveEvent(QMouseEvent *event)
{
	
	//if (event->button() == Qt::LeftButton/* && b_drug*/) {
		move(0,event->globalY());
		if(m_panel){
		    m_panel->resize(i_width, i_height-event->globalY()-52);
		    m_panel->setMinimumSize(i_width, i_height-event->globalY()-52);
		    m_panel->setMaximumSize(i_width, i_height-event->globalY()-52);
		}
	//}
}


void ConfigPanel::setPanel(Panel *p)
{
    m_panel = p;
}


void ConfigPanel::focusOutEvent ( QFocusEvent * event )
{
    close();
}

void ConfigPanel::onActiveWindowChanged(WId id)
{
    if(id != winId()){
	close();
    }
}

#include "configpanel.moc"

