/***************************************************************************
 *   Copyright 2007 by Marco Martin <notmart@gmail.com>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "separator.h"

//QT
#include <QGraphicsSceneDragDropEvent>
#include <QGraphicsLinearLayout>

//KDE
#include <KCModuleProxy>
#include <KConfigDialog>
#include <KDebug>
#include <KLocale>
#include <KIconLoader>
#include <KRun>
#include <KSharedConfig>
#include <KMessageBox>
#include <KUrl>
#include <KProcess>
#include <KStandardDirs>
#include <KNotification>
#include <KGlobalSettings>

#include <kfileplacesmodel.h>

#include <KIO/CopyJob>
#include <KIO/JobUiDelegate>

//Plasma
#include <Plasma/IconWidget>
#include <Plasma/Containment>
#include <Plasma/ToolTipManager>
#include <Plasma/WindowEffects>


//Solid
#include <solid/devicenotifier.h>
#include <solid/device.h>
#include <solid/deviceinterface.h>
#include <solid/predicate.h>
#include <solid/storageaccess.h>
#include <solid/opticaldrive.h>
#include <solid/opticaldisc.h>

#include <plasma/svg.h>
#include <plasma/theme.h>


RosaSeparator::RosaSeparator(QObject *parent, const QVariantList &args):
    Plasma::Applet(parent, args),
    m_svg(this)
{
    m_svg.setImagePath("widgets/panelspacer-separator"); //("widgets/tasks.svgz");//("panelspacer-separator");
    m_svg.setContainsMultipleImages(true);

    setHasConfigurationInterface(false);
    //setAspectRatioMode(Plasma::Square);
}

RosaSeparator::~RosaSeparator()
{
}

void RosaSeparator::init()
{
    setAcceptDrops(false);
    //setAspectRatioMode(Plasma::Square);

	QRectF sepRect = m_svg.elementRect("separator-1-horizontal");

    m_svgRatio = sepRect.width() / sepRect.height();

    setMinimumWidth(15);
    setMaximumWidth(9999);
    setMinimumHeight(15);
    setMaximumHeight(9999);

    connect(this, SIGNAL(geometryChanged()), this, SLOT(updateSizes()));
}

void RosaSeparator::updateSizes(void)
{
  if(formFactor() == Plasma::Horizontal)
    {
      //kDebug() << "FACTOR IS HORIZONTAL";
      setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);

      float height = geometry().height();
      float width = height * m_svgRatio;
      
      setMinimumWidth(width);
      setMaximumWidth(width);
      setMinimumHeight(15);
      setMaximumHeight(9999);
    }
  else if(formFactor() == Plasma::Vertical)
    {
      //kDebug() << "FACTOR IS VERTICAL";
      setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);

      float width = geometry().width();
      float height = width * m_svgRatio;
      
      setMinimumWidth(15);
      setMaximumWidth(9999);
      setMinimumHeight(height);
      setMaximumHeight(height);
    }

  update();
}

void RosaSeparator::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{

  p->save();
  
	//setMinimumHeight(size().height());
	//setMinimumHeight(size().height());

	p->setRenderHint(QPainter::SmoothPixmapTransform);
	p->setRenderHint(QPainter::Antialiasing);
	
	
	QRectF sepRect = m_svg.elementRect("separator-1-horizontal");

/*	
	m_svg.resize(size().height(),size().height());
	m_svg.paint(p, 5,contentsRect.y()-sepRect.y(),size().height(),size().height(),"separator-1-horizontal");
	//m_svg.paint(p,(200 - 150) / 2,200 - 150,175,50);//,"glowing-dot");
*/	
/*
	if(location() == Plasma::LeftEdge){
      //setMinimumHeight(15);
      //setMaximumHeight(15);

		if(Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::WindowPreview)){
		    m_svg.paint(p, 1,size().height(),contentsRect.y()-sepRect.y()+12,size().width(),"separator-1-vertical");
		}else{
		    m_svg.paint(p, 1,size().height(),contentsRect.y()-sepRect.y()+5,size().width(),"separator-1-vertical");
		}
		m_svg.resize(size().width(),size().height());

	}else if(location() == Plasma::RightEdge){
      //setMinimumHeight(15);
      //setMaximumHeight(15);
		
		if(Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::WindowPreview)){
		    m_svg.paint(p, 1,size().height(),contentsRect.y()-sepRect.y()+12,size().width(),"separator-2-vertical");
		}else{
		    m_svg.paint(p, 1,size().height(),contentsRect.y()-sepRect.y()+5,size().width(),"separator-2-vertical");
		}
		m_svg.resize(size().width(),size().height());
	}else{
      //setMinimumWidth(15);
      //setMaximumWidth(15);

		if(Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::WindowPreview)){
		    m_svg.paint(p, 1,contentsRect.y()-sepRect.y()+12,size().width(),size().height(),"separator-1-horizontal");
		}else{
		    m_svg.paint(p, 1,contentsRect.y()-sepRect.y()+5,size().width(),size().height(),"separator-1-horizontal");
		}
		m_svg.resize(size().height(),size().width());
	}
*/

    if(formFactor() == Plasma::Vertical)
      {
        //kDebug() << "Drawing Vertical";

        if(Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::WindowPreview)){
          m_svg.paint(p, contentsRect.x() - sepRect.x(), 1, size().width(), size().height(), "separator-1-vertical");
        }else{
          m_svg.paint(p, contentsRect.x() - sepRect.x(), 1, size().width(), size().height(), "separator-1-vertical");
        }
        m_svg.resize(size().width(),size().height());

      }
    else // Horizontal or others
      {
        //kDebug() << "Drawing Horizontal";
        
		if(Plasma::WindowEffects::isEffectAvailable(Plasma::WindowEffects::WindowPreview)){
          //m_svg.paint(p, 1, contentsRect.y() - sepRect.y() + 12, size().width(), size().height(), "separator-1-horizontal");
          m_svg.paint(p, 1, contentsRect.y() - sepRect.y(), size().width(), size().height(), "separator-1-horizontal");
		}else{
          //m_svg.paint(p, 1, contentsRect.y() - sepRect.y() + 5, size().width(), size().height(), "separator-1-horizontal");
          m_svg.paint(p, 1, contentsRect.y() - sepRect.y(), size().width(), size().height(), "separator-1-horizontal");
		}
		m_svg.resize(size().height(),size().width());
      }

    
	p->restore();
}

#include "separator.moc"

